#!/usr/bin/perl

# $Id: configtest.pl 1156 2007-01-05 06:36:07Z rjl $

########################################################################
# MAIA MAILGUARD LICENSE v.1.0
#
# Copyright 2004 by Robert LeBlanc <rjl@renaissoft.com>
#                   David Morton   <mortonda@dgrmm.net>
# All rights reserved.
#
# PREAMBLE
#
# This License is designed for users of Maia Mailguard
# ("the Software") who wish to support the Maia Mailguard project by
# leaving "Maia Mailguard" branding information in the HTML output
# of the pages generated by the Software, and providing links back
# to the Maia Mailguard home page.  Users who wish to remove this
# branding information should contact the copyright owner to obtain
# a Rebranding License.
#
# DEFINITION OF TERMS
#
# The "Software" refers to Maia Mailguard, including all of the
# associated PHP, Perl, and SQL scripts, documentation files, graphic
# icons and logo images.
#
# GRANT OF LICENSE
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. The end-user documentation included with the redistribution, if
#    any, must include the following acknowledgment:
#
#    "This product includes software developed by Robert LeBlanc
#    <rjl@renaissoft.com>."
#
#    Alternately, this acknowledgment may appear in the software itself,
#    if and wherever such third-party acknowledgments normally appear.
#
# 4. At least one of the following branding conventions must be used:
#
#    a. The Maia Mailguard logo appears in the page-top banner of
#       all HTML output pages in an unmodified form, and links
#       directly to the Maia Mailguard home page; or
#
#    b. The "Powered by Maia Mailguard" graphic appears in the HTML
#       output of all gateway pages that lead to this software,
#       linking directly to the Maia Mailguard home page; or
#
#    c. A separate Rebranding License is obtained from the copyright
#       owner, exempting the Licensee from 4(a) and 4(b), subject to
#       the additional conditions laid out in that license document.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
########################################################################

    use DBI;

    # prototypes
    sub get_module_version($);
    sub print_message($$$);
    sub is_numeric($);
    sub decode_version_string($);

    # name of this script
    my $script_name = "configtest";

    # read configuration file (/etc/maia.conf)
    my $config_file = "/etc/maia.conf";
    unless (my $rv = do $config_file) {
        die("Maia: [$script_name] Couldn't parse $config_file: $@\n") if $@;
        die("Maia: [$script_name] Couldn't open $config_file\n") if (!defined($rv) || !$rv);
    };

    my @components = (
        {component => "Perl", type => "perl", regexp => "", minver => "5.008003",
         reason => "should never see this!"},

        {component => "file(1)", type => "file -v 2>/dev/null", regexp => "\-([0-9\.]+)", minver => "4.12",
         reason => "required by Maia Mailguard"},

        {component => "Archive::Tar", type => "module", regexp => "", minver => "",
         reason => "required by Maia Mailguard"},

        {component => "Archive::Zip", type => "module", regexp => "", minver => "1.14",
         reason => "required by Maia Mailguard"},

        {component => "BerkeleyDB", type => "module", regexp => "", minver => "",
         reason => "Maia Mailguard's optional caching feature requires this"},

        {component => "Compress::Zlib", type => "module", regexp => "", minver => "1.35",
         reason => "required by Maia Mailguard"},

        {component => "Convert::TNEF", type => "module", regexp => "", minver => "",
         reason => "required by Maia Mailguard"},

        {component => "Convert::UUlib", type => "module", regexp => "", minver => "1.06",
         reason => "required by Maia Mailguard"},

        {component => "Crypt::Blowfish", type => "module", regexp => "", minver => "",
         reason => "Maia Mailguard's optional encryption feature requires this"},

        {component => "Crypt::CBC", type => "module", regexp => "", minver => "2.22",
         reason => "Maia Mailguard's optional encryption feature requires this"},

        {component => "Crypt::OpenSSL::RSA", type => "module", regexp => "", minver => "",
         reason => "SpamAssassin's optional DomainKeys plugin requires this"},

        {component => "Data::UUID", type => "module", regexp => "", minver => "",
         reason => "required by Maia Mailguard"},

        {component => "DB_File", type => "module", regexp => "", minver => "",
         reason => "required by SpamAssassin"},

        {component => "DBD::mysql", type => "module", regexp => "", minver => "2.9003",
         reason => "required if you use MySQL as your Maia Mailguard database"},

        {component => "DBD::Pg", type => "module", regexp => "", minver => "1.31",
         reason => "required if you use PostgreSQL as your Maia Mailguard database"},

        {component => "DBI", type => "module", regexp => "", minver => "1.40",
         reason => "required by Maia Mailguard and SpamAssassin"},

        {component => "Digest::MD5", type => "module", regexp => "", minver => "2.22",
         reason => "required by Maia Mailguard"},

        {component => "Digest::SHA1", type => "module", regexp => "", minver => "",
         reason => "required by SpamAssassin"},

        {component => "File::Spec", type => "module", regexp => "", minver => "0.80",
         reason => "required by SpamAssassin"},

        {component => "HTML::Parser", type => "module", regexp => "", minver => "3.31",
         reason => "required by SpamAssassin"},

        {component => "HTTP::Date", type => "module", regexp => "", minver => "",
         reason => "SpamAssassin's sa-update script requires this"},

        {component => "IO::Stringy", type => "module", regexp => "", minver => "",
         reason => "required by Maia Mailguard"},

        {component => "IO::Zlib", type => "module", regexp => "", minver => "",
         reason => "SpamAssassin's sa-update script requires this"},

        {component => "IP::Country", type => "module", regexp => "", minver => "",
         reason => "SpamAssassin's optional RelayCountry plugin requires this"},

        {component => "LWP::UserAgent", type => "module", regexp => "", minver => "",
         reason => "SpamAssassin's sa-update script requires this"},

        {component => "Mail::Address", type => "module", regexp => "", minver => "",
         reason => "SpamAssassin's optional DomainKeys plugin requires this"},

        {component => "Mail::DomainKeys", type => "module", regexp => "", minver => "",
         reason => "SpamAssassin's optional DomainKeys plugin requires this"},

        {component => "Mail::Internet", type => "module", regexp => "", minver => "1.58",
         reason => "required by Maia Mailguard"},

        {component => "Mail::SpamAssassin", type => "module", regexp => "", minver => "3.001007",
         reason => "required by Maia Mailguard"},

        {component => "Mail::SPF::Query", type => "module", regexp => "", minver => "", 
         reason => "SpamAssassin's optional SPF plugin requires this"},

        {component => "MIME::Base64", type => "module", regexp => "", minver => "",
         reason => "required by Maia Mailguard and SpamAssassin"},

        {component => "MIME::Parser", type => "module", regexp => "", minver => "5.420",
         reason => "required by Maia Mailguard"},

        {component => "MIME::QuotedPrint", type => "module", regexp => "", minver => "3.03",
         reason => "required by Maia Mailguard"},

        {component => "Net::CIDR::Lite", type => "module", regexp => "", minver => "",
         reason => "SpamAssassin's optional SPF plugin requires this"},

        {component => "Net::DNS", type => "module", regexp => "", minver => "0.34",
         reason => "required by SpamAssassin"},

        {component => "Net::Server", type => "module", regexp => "", minver => "0.93",
         reason => "required by Maia Mailguard"},

        {component => "Net::SMTP", type => "module", regexp => "", minver => "",
         reason => "required by Maia Mailguard"},

        {component => "Pod::Usage", type => "module", regexp => "", minver => "1.10",
         reason => "required by SpamAssassin"},

        {component => "Template", type => "module", regexp => "", minver => "2.10",
         reason => "required by Maia Mailguard"},

        {component => "Time::HiRes", type => "module", regexp => "", minver => "1.49",
         reason => "required by Maia Mailguard"},

        {component => "Unix::Syslog", type => "module", regexp => "", minver => "",
         reason => "required by Maia Mailguard"},

        {component => "URI", type => "module", regexp => "", minver => "",
         reason => "required by SpamAssassin"},

    );      

    print "\nMAIA MAILGUARD CONFIGURATION TEST\n\n";

    print "This script checks for the presence of applications and Perl modules\n";
    print "required by amavisd-maia, SpamAssassin, and Maia Mailguard's maintenance\n";
    print "scripts.  Version numbers are also checked, and if a newer version of\n";
    print "a component is recommended, you should consider upgrading to at least\n";
    print "the minimum recommended version.\n\n";

    print "If you have already configured your Maia Mailguard database, the script\n";
    print "will also test the connection to that database.\n\n";

    print "Remember also to run the configtest.php script on your web server to\n";
    print "perform similar tests of your web, PHP, and PEAR environment.\n\n";

    print "Application/Module      Version   Status\n";
    print "========================================================================\n";

    my $have_dbi = 0;
    my $have_dbd_mysql = 0;
    my $have_dbd_pg = 0;
    my $item;

    foreach $item (@components) {
        my %hash = %$item;
        my $component = $hash{"component"};
        my $type = $hash{"type"};
        my $regexp = $hash{"regexp"};
        my $min_version = $hash{"minver"};
        my $reason = $hash{"reason"};
        my $version;
        my $upgrade = 0;
        
        # it's a Perl module, query its version number.
        if ($type eq "module") {
            $version = get_module_version($component);

        # it's Perl itself, so determine its version number directly.
        } elsif ($type eq "perl") {
            $version = $];

        # it's an external app, invoke its version-printing command
        # and grab the result with an app-specific regular expression.
        } else {
            $version = `$type`;
            if ($regexp ne "") {
                $version = (($version =~ /$regexp/si) ? $1 : undef);
            }
        }

        # is an upgrade recommended?
        if (defined $version && $min_version ne "") {
            if (is_numeric($version) && is_numeric($min_version)) {
                $upgrade = $version < $min_version;
            } else {
                $upgrade = $version lt $min_version;
            }
        }

        if (defined $version) {
            $version = decode_version_string($version);
            $min_version = decode_version_string($min_version);
            if ($upgrade) {
                print_message($component, $version, "UPGRADE RECOMMENDED (minimum version $min_version)");
            } else {
                print_message($component, $version, "OK");
            }
            $have_dbi = 1 if ($component eq "DBI");
            $have_dbd_mysql = 1 if ($component eq "DBD::mysql");
            $have_dbd_pg = 1 if ($component eq "DBD::Pg");
        } else {
            print_message($component, "N/A", "NOT INSTALLED ($reason)");
        }
    }

    print("\n");

    if ($have_dbi && ($have_dbd_mysql || $have_dbd_pg)) {

        my $dbh;

        # database configuration
        if (defined($dsn) && defined($username) && defined($password)) {
            $dbh = DBI->connect($dsn, $username, $password)
                or die("Database DSN test    : FAILED " .
                       "(verify \$dsn, \$username, and \$password in $config_file)\n\n");                       
        } else {
            die("Database DSN test    : FAILED " .
                "(missing \$dsn, \$username, or \$password in $config_file)\n\n");
        }

        print("Database DSN test    : PASSED\n");

        # Disconnect from the database
        $dbh->disconnect;

    } else {
    	print("Database tests SKIPPED (missing DBI and/or DBD modules)\n");
    }

    print "\n";

    exit;


    # Returns the version number of the named module, or undef if the module
    # is not installed.
    sub get_module_version($) {
        my($module) = @_;

        my $version = `$^X -m$module -e 'print \$${module}::VERSION' 2>/dev/null`;
        $version =~ s/^\s*(.*?)\s*$/$1/;
        return ($version || undef);
    }


    # Prints a formatted text message.
    sub print_message($$$) {
        my($module, $version, $message) = @_;

        printf("%-20s : %8s : %s\n", $module, $version, $message);
    }

    # Returns true if a string is a valid integer or floating point value.
    sub is_numeric($) {
        my ($string) = @_;

        return ($string =~ /^([0-9]*\.*[0-9]+)$/);
    }

    # Convert standard M.mmmttt version numbers to M.m.t format 
    # for display purposes.
    sub decode_version_string($) {
        my ($version) = @_;
        my ($major, $minor, $trivial);

        if ($version =~ /^([0-9]+)\.([0-9]{3})([0-9]{3})$/) {
           $major = int($1);
           $minor = int($2);
           $trivial = int($3);
           return ($major . "." . $minor . "." . $trivial);
        } else {
           return ($version);
        }
    }
